==> kari-0.24.tar.gz: kari-0.24/parser/attrib_utils.c <==
#include <string.h>
#include <ctype.h>
#include <stdio.h>

#include "parse_attribs.h"
#include "attrib_utils.h"
#include "utils.h"

int get_bit (char *document, long *index)
{
  int temp = 0;
  foo[9] = '\0';
  
  if (strlen(document+(*index)))
    {
      strncpy (foo, document+(*index), 
	       temp = strlen(document+(*index)) < 9 ? temp : 9);
      return 1;
    }
  else
    return 0;
}



int skip_equals(char *document, long *index) 
{
  /* Skips optional whitespace followed by an '=' followed by more
   * optional whitespace.
   */

  debug (debug_parse_attribs, "skip_equals: entering pointing at %ld:%s...", 
	 *index, get_bit(document, index) ? foo  : "NULL");
  
  while (is_whitespace(document[*index]))
    (*index)++;
  
  if (document[*index]!='=')
    return -1;
  
  (*index)++;
  
  while (is_whitespace(document[*index]))
    (*index)++;
  
  debug (debug_parse_attribs, "skip_equals: leaving pointing at %ld:%s...",      
	 *index, get_bit (document, index) ? foo : "NULL");

  
  return 0;
}



long handle_value (char *document, long *index)
{
  /* Convert a string of numbers to their decimal value, just keeps
   * going until we hit a non-digit. YARK This is probably
   * very evil, but it should work in 99% of vaguely sane cases
   */
  long running_total = 0;
  
  debug (debug_parse_attribs, "handle_value: Entering pointing at %ld:%s...", 
	 *index, get_bit (document, index) ? foo : "NULL");
  
  while (isdigit(document[*index])) 
    {
      running_total *= 10;
      running_total += document[*index] - '0'; /* Safe for ASCII, should be others */
      debug (debug_parse_attribs, "handle_value: Running Total = %ld.", running_total);
      (*index)++;
    }
  
  debug ( debug_parse_attribs, "handle_value: Returning %ld, now pointing at %ld:%s...",
	  running_total, *index, get_bit (document, index) ? foo : "NULL");
  
  return running_total;
}



int hex_to_dec (char s)
{
  /* Returns the decimal value of a hex digit */
  if (isdigit(s))
    return s-'0';
  else if (s >= 'a' && s <= 'f')
    return 10+s-'a';
  else if (s >= 'A' && s <= 'F')
    return 10+s-'A';
  return -1; /* Keep gcc happy */
}



int convert_hex (char *doc, long *index)
{
  /* Return the hex value of a two character sequence provided they
   * are both valid hex digits
   */
  int value;
  
  if (isxdigit(doc[*index]) && isxdigit(doc[*index+1])) {
    
    value = 16 * (hex_to_dec(doc[*index]));
    (*index)++;
    
    value += hex_to_dec(doc[*index]);
    (*index)++;
    
    return value;
  
  }
  
  else
  
    return -1;
}



void skip_attribs (char *document, long *index)
{
  /* Skips past attriutes which we don't recognize, it'll try to cope sensibly with
   * various different forms and terminate pointing at the right place.
   */
  
  debug (debug_parse_attribs, "skip_attribs: Entered pointing at %ld:%s",
	 *index, get_bit (document, index) ? foo : "NULL");
  do 
    {
      /* Skip until we find a close tag, EOF or a quote (Which we'll
       * presume is the first of a pair
       */
      while (document[*index] != '\0' && document[*index] != '>' && document[*index]!= '"')
	(*index)++;
      
      if (document[*index] != '\0' && document[*index] == '"')
	{
	  (*index)++; /* Skip opening " */
	  
	  /* Presume we're in quoted section so ignore everything
	   * until we find a close quote
	   */
	  while (document[*index] != '\0' && document[*index] != '"')
	    (*index)++;
	  
	  if (document[*index] != '\0')
	    (*index)++; /* Skip closing */
	}
    
    } while (document[*index] != '\0' && document[*index] != '>');
  
  if (document[*index] != '\0')
    (*index)++;

  debug ( debug_parse_attribs, "skip_attribs: Leaving pointing at %ld:%s",
	  *index, get_bit (document, index) ? foo : "NULL");
}



void fill_in_alt(neve_in_chains *node)
{
  if (!node->src)
    {
      /* We don't have any src info either, let's let them know their
       * HTML is more broken than a very broken thing.
       */
      const char *error_msg = "{IMAGE:Broken tag, abuse this person ...}";
      node->buf = xmalloc (strlen(error_msg)+1);
      strcpy(node->buf, error_msg);
    }  
  else
    {
      /* If we have a specified URL for the image but no alt text we'll strip the
       * actual filename from the end of the URL and use that since
       * everyone needs some alt text.
       */
      int hpointer; /* Record the position we're skipping back in in the SRC */
      int acounter = 0; /*Record the number of chars we'll need in the node->buf */
      
      hpointer = strlen(node->src)-1;
      acounter = 1; /* We'll always need a '/0' in addition to the text */
      
      debug (debug_parse_attribs, "fill_in_alt: Node's SRC is %s.", node->src);

      while (hpointer  && node->src[hpointer] != '/')
	{
	  debug (debug_parse_attribs, "fill_in_alt: hpointer = %d(%c)", 
		 hpointer, node->src[hpointer]);
	  hpointer--;
	  debug (debug_parse_attribs, "fill_in_alt: hpointer = %d", hpointer);
	}
      
      if (hpointer != 0 || (node->src[hpointer] == '/' && hpointer == 0))
	{
	  hpointer++;
	}
      
      debug ( debug_parse_attribs, "fill_in_alt: hpointer = %d -> %s", 
	      hpointer, node->src+hpointer );
      
      node->buf = xmalloc (strlen(node->src+hpointer)+9);
      sprintf (node->buf, "{IMAGE:%s}", node->src+hpointer);
      
      debug ( debug_parse_attribs, "node->buf = %s", node->buf ) ;
    }
}

==> kari-0.30.tar.gz: kari-0.30/parser/attrib_utils.c <==
/* YAWN, Yet Another Web Navigator */
/* Copyright (C) 1998, James Troup <jjtroup@comp.brad.ac.uk>
 *                     Lee Willis <lwillis@comp.brad.ac.uk> */

/* This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <string.h>
#include <ctype.h>
#include <stdio.h>

#include "parse_attribs.h"
#include "attrib_utils.h"
#include "utils.h"

int get_bit (const char *document, long *index)
{
  int temp = 0;
  foo[9] = '\0';
  
  if (strlen(document+(*index)))
    {
      strncpy (foo, document+(*index), 
	       temp = strlen(document+(*index)) < 9 ? temp : 9);
      return 1;
    }
  else
    return 0;
}



int skip_equals(const char *document, long *index) 
{
  /* Skips optional whitespace followed by an '=' followed by more
   * optional whitespace.
   */

  debug (debug_parse_attribs, "skip_equals: entering pointing at %ld:%s...", 
	 *index, get_bit(document, index) ? foo  : "NULL");
  
  while (is_whitespace(document[*index]))
    (*index)++;
  
  if (document[*index]!='=')
    return -1;
  
  (*index)++;
  
  while (is_whitespace(document[*index]))
    (*index)++;
  
  debug (debug_parse_attribs, "skip_equals: leaving pointing at %ld:%s...",      
	 *index, get_bit (document, index) ? foo : "NULL");

  
  return 0;
}



long handle_value (const char *document, long *index)
{
  /* Convert a string of numbers to their decimal value, just keeps
   * going until we hit a non-digit. YARK This is probably
   * very evil, but it should work in 99% of vaguely sane cases
   */
  long running_total = 0;
  
  debug (debug_parse_attribs, "handle_value: Entering pointing at %ld:%s...", 
	 *index, get_bit (document, index) ? foo : "NULL");
  
  while (isdigit(document[*index])) 
    {
      running_total *= 10;
      running_total += document[*index] - '0'; /* Safe for ASCII, should be others */
      debug (debug_parse_attribs, "handle_value: Running Total = %ld.", running_total);
      (*index)++;
    }
  
  debug ( debug_parse_attribs, "handle_value: Returning %ld, now pointing at %ld:%s...",
	  running_total, *index, get_bit (document, index) ? foo : "NULL");
  
  return running_total;
}



int hex_to_dec (char s)
{
  /* Returns the decimal value of a hex digit */
  if (isdigit(s))
    return s-'0';
  else if (s >= 'a' && s <= 'f')
    return 10+s-'a';
  else if (s >= 'A' && s <= 'F')
    return 10+s-'A';
  return -1; /* Keep gcc happy */
}



int convert_hex (const char *document, long *index)
{
  /* Return the hex value of a two character sequence provided they
   * are both valid hex digits
   */
  int value;
  
  if (isxdigit(document[*index]) && isxdigit(document[*index+1])) {
    
    value = 16 * (hex_to_dec(document[*index]));
    (*index)++;
    
    value += hex_to_dec(document[*index]);
    (*index)++;
    
    return value;
  
  }
  
  else
  
    return -1;
}



void skip_attribs (const char *document, long *index)
{
  /* Skips past attriutes which we don't recognize, it'll try to cope sensibly with
   * various different forms and terminate pointing at the right place.
   */
  
  debug (debug_parse_attribs, "skip_attribs: Entered pointing at %ld:%s",
	 *index, get_bit (document, index) ? foo : "NULL");
  do 
    {
      /* Skip until we find a close tag, EOF or a quote (Which we'll
       * presume is the first of a pair
       */
      while (document[*index] != '\0' && document[*index] != '>' && document[*index]!= '"')
	(*index)++;
      
      if (document[*index] != '\0' && document[*index] == '"')
	{
	  (*index)++; /* Skip opening " */
	  
	  /* Presume we're in quoted section so ignore everything
	   * until we find a close quote
	   */
	  while (document[*index] != '\0' && document[*index] != '"')
	    (*index)++;
	  
	  if (document[*index] != '\0')
	    (*index)++; /* Skip closing */
	}
    
    } while (document[*index] != '\0' && document[*index] != '>');
  
  if (document[*index] != '\0')
    (*index)++;

  debug ( debug_parse_attribs, "skip_attribs: Leaving pointing at %ld:%s",
	  *index, get_bit (document, index) ? foo : "NULL");
}



void fill_in_alt(neve_in_chains *node)
{
  if (!node->src)
    {
      /* We don't have any src info either, let's let them know their
       * HTML is more broken than a very broken thing.
       */
      const char *error_msg = "{IMAGE:Broken tag, abuse this person ...}";
      node->buf = xmalloc (strlen(error_msg)+1);
      strcpy(node->buf, error_msg);
    }  
  else
    {
      /* If we have a specified URL for the image but no alt text we'll strip the
       * actual filename from the end of the URL and use that since
       * everyone needs some alt text.
       */
      int hpointer; /* Record the position we're skipping back in in the SRC */
      int acounter = 0; /*Record the number of chars we'll need in the node->buf */
      
      hpointer = strlen(node->src)-1;
      acounter = 1; /* We'll always need a '/0' in addition to the text */
      
      debug (debug_parse_attribs, "fill_in_alt: Node's SRC is %s.", node->src);

      while (hpointer  && node->src[hpointer] != '/')
	{
	  debug (debug_parse_attribs, "fill_in_alt: hpointer = %d(%c)", 
		 hpointer, node->src[hpointer]);
	  hpointer--;
	  debug (debug_parse_attribs, "fill_in_alt: hpointer = %d", hpointer);
	}
      
      if (hpointer != 0 || (node->src[hpointer] == '/' && hpointer == 0))
	{
	  hpointer++;
	}
      
      debug ( debug_parse_attribs, "fill_in_alt: hpointer = %d -> %s", 
	      hpointer, node->src+hpointer );
      
      node->buf = xmalloc (strlen(node->src+hpointer)+9);
      sprintf (node->buf, "{IMAGE:%s}", node->src+hpointer);
      
      debug ( debug_parse_attribs, "node->buf = %s", node->buf ) ;
    }
}
